# Generated by Django 2.2.7 on 2020-10-31 02:46

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('openacct', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='default_project',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='openacct.Project', blank=True, null=True, related_name='+'),
        ),
    ]

