__version__     = "0.0.5"

__title__       = "django-openacct"
__description__ = "A pluggable Django app for managing research computing center project accounting."
__uri__         = "https://github.com/Elemnir/django-openacct"

__author__      = "Adam Howard"
__email__       = "ahoward@utk.edu"

__license__     = "BSD 3-clause"
__copyright__   = "Copyright (c) 2020 Adam Howard"


